"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
class Rpc {
    constructor(ws, opt = {}) {
        this.ws = ws;
        this.ws.on("message", async (str) => {
            try {
                await this.handleMessage(str);
            }
            catch (e) {
                console.error(e);
            }
        });
        this.commands = {};
        this.requests = {};
        this.options = opt;
    }
    async handleMessage(str) {
        let msg;
        try {
            msg = typeof str === "string" ? JSON.parse(str) : str;
        }
        catch (e) {
            throw new Error("MESSAGE_ERROR");
        }
        if (msg.type === "command") {
            let command = this.commands[msg.command];
            if (!command) {
                this.sendMessage({ type: "reply", id: msg.id, status: "error", error: "UNKNOWN_COMMAND" });
            }
            else {
                try {
                    let args = msg.args || [];
                    let result = command.handler(...msg.args);
                    if (result && result.then) {
                        result = await result;
                    }
                    this.sendMessage({ type: "reply", id: msg.id, status: "ok", result });
                }
                catch (e) {
                    this.sendMessage({ type: "reply", id: msg.id, status: "error", error: e && e.toString() });
                }
            }
        }
        else if (msg.type === "reply") {
            let req = this.requests[msg.id];
            if (!req) {
                throw new Error("UNKNOWN_REPLY");
            }
            clearTimeout(req.timeout);
            if (msg.status == "ok") {
                req.resolve(msg.result);
            }
            else {
                req.reject(new Error(msg.error));
            }
            delete this.requests[msg.id];
        }
        else {
            throw new Error("UNKNOWN_TYPE");
        }
    }
    sendMessage(payload) {
        this.ws.send(JSON.stringify(payload));
    }
    async exec(command, args = []) {
        let msgId = uuid_1.v4();
        return new Promise((resolve, reject) => {
            this.sendMessage({
                id: msgId,
                type: "command",
                command,
                args
            });
            let timeout = setTimeout(() => {
                if (this.requests[msgId]) {
                    this.requests[msgId].reject(new Error("EXEC_TIMEOUT"));
                }
            }, this.options.timeout || 5000);
            this.requests[msgId] = {
                reject,
                resolve,
                timeout
            };
        });
    }
    createCommand(command, handler) {
        this.commands[command] = {
            handler
        };
    }
    removeCommand(command) {
        delete this.commands[command];
    }
}
exports.default = Rpc;
//# sourceMappingURL=rpc.js.map