"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const merapi_1 = require("merapi");
const WebSocket = require("ws");
const rpc_1 = require("./rpc");
const sleep = require("then-sleep");
class SvxWebsocket extends merapi_1.Component {
    constructor(config, logger, service) {
        super();
        this.config = config;
        this.logger = logger;
        this.service = service;
        this._moduleName = "websocket";
        this._priority = 3;
        this._version = 1;
        this._supportTransport = {
            api: true,
            command: true,
            topic: true,
            queue: true,
            discovery: true,
            log: true
        };
    }
    async initialize() {
        this.service.setDefaultTransport(this);
        await this.service.registerModule(this);
        return this.doConnect();
    }
    async doConnect() {
        const websocketUrl = this.config.default("websocket.url", "ws://localhost:8000");
        this._connection = new rpc_1.default(new WebSocket(websocketUrl), this.config.default("websocket.options", {}));
    }
    async start() {
        this._connection.createCommand("discovery.discover", async () => {
            this.broadcast(this.service.info());
        });
        this.handleBroadcast();
        setInterval(async () => {
            this.broadcast(this.service.info());
        }, parseInt(this.config.default("websocket.broadcastInterval", "5000")));
    }
    name() {
        return this._moduleName;
    }
    priority() {
        return this._priority;
    }
    info() {
        return {
            version: this._version,
            transport: this._supportTransport
        };
    }
    hasTransport(type) {
        if (Object.keys(this._supportTransport).indexOf(type) > -1)
            return this._supportTransport[type];
        return false;
    }
    getConnection() {
        return this._connection;
    }
    async publishTopic(service, topic, version, payload, source) {
        this._connection.exec("publishTopic", [service, topic, version, payload, source]);
    }
    subscribeTopic(service, topic, version, handler) {
        let source = this.service.createSource();
        this._connection.createCommand(`subscribeTopic.${service}.${topic}.${version}`, handler);
        this._connection.exec(`createTopic`, [service, topic, version]);
    }
    unsubscribeTopic(service, topic, version) {
        this._connection.exec(`unsubscribeTopic`, [service, topic, version]);
        this._connection.removeCommand(`subscribeTopic.${service}.${topic}.${version}`);
    }
    pushQueue(service, queue, version, payload, source) {
        this._connection.exec("pushQueue", [service, queue, version, payload, source]);
    }
    consumeQueue(service, queue, version, handler) {
        let source = this.service.createSource();
        this._connection.createCommand(`consumeQueue.${service}.${queue}.${version}`, handler);
        this._connection.exec(`createQueue`, [service, queue, version]);
    }
    async callApi(service, method, version, args, source) {
        return await this._connection.exec("callApi", [service, method, version, args, source]);
    }
    createApi(service, method, version, handler) {
        this._connection.createCommand(`handleApi.${service}.${method}.${version}`, handler);
        this._connection.exec("createApi", [service, method, version]);
    }
    async watchApiCall(service, method, version) {
        return;
    }
    async watchApiResult(service, method, version) {
        return;
    }
    async execCommand(service, command, args, source) {
        return await this._connection.exec("execCommand", [service, command, args, source]);
    }
    createCommand(service, command, handler) {
        this._connection.createCommand(`handleCommand.${service}.${command}`, handler);
        this._connection.exec("createCommand", [service, command]);
    }
    handleBroadcast() {
        this._connection.createCommand("discovery.broadcast", (info, source) => {
            this.service.handleBroadcast(info, source);
        });
    }
    async broadcast(info) {
        this._connection.exec("discovery.broadcast", [info, this.service.createSource()]);
    }
    async discover(timeout = 500) {
        this._connection.exec("discovery.discover", [{}]);
        return new Promise((resolve, reject) => {
            setTimeout(resolve, timeout);
        });
    }
    subscribeLog(service, level, handler) {
        let source = this.service.createSource();
        this._connection.exec("subscribeLog", [service, level]);
        this._connection.createCommand(`handleLog.${service}.${level}`, handler);
    }
    log(service, component, method, level, message) {
        let source = this.service.createSource();
        this._connection.exec("publishLog", [service, component, method, level, message, source]);
    }
}
exports.default = SvxWebsocket;
//# sourceMappingURL=svx_websocket.js.map