"use strict";

import { JsonObject } from "merapi";
import { IRpc } from "./interface";

import * as WebSocket from "ws";
import { v4 as uuid } from "uuid";

export default class Rpc implements IRpc {
    private ws : WebSocket;
    private commands : { [key : string] : { handler: Function } };
    private requests : { [key : string] : { reject: any, resolve: any, timeout: any }};
    private options : JsonObject;

    constructor(ws : WebSocket, opt = {}) {
        this.ws = ws;
        this.ws.on("message", async (str : string | JsonObject) => {
            try {
                await this.handleMessage(str);
            } catch(e) {
                console.error(e);
            }
        });
        this.commands = {};
        this.requests = {};
        this.options = opt;
    }

    async handleMessage(str : string | JsonObject) : Promise<void> {
        let msg;
        try {
            msg = typeof str === "string" ? JSON.parse(str) : str;
        } catch(e) {
            throw new Error("MESSAGE_ERROR");
        }

        if (msg.type === "command") {
            let command = this.commands[msg.command];
            if (!command) {
                this.sendMessage({ type: "reply", id: msg.id, status: "error", error: "UNKNOWN_COMMAND" });
            } else {
                try {
                    let args = msg.args || [];
                    let result = command.handler(...msg.args);
                    if (result && result.then) {
                        result = await result;
                    }
                    this.sendMessage({ type: "reply", id: msg.id, status: "ok", result });
                } catch(e) {
                    this.sendMessage({ type: "reply", id: msg.id, status: "error", error: e && e.toString() });
                }
            }
        } else if (msg.type === "reply") {
            let req = this.requests[msg.id];
            if (!req) {
                throw new Error("UNKNOWN_REPLY");
            }
            clearTimeout(req.timeout);
            if (msg.status == "ok") {
                req.resolve(msg.result);
            } else {
                req.reject(new Error(msg.error));
            }
            delete this.requests[msg.id];
        } else {
            throw new Error("UNKNOWN_TYPE");
        }
    }

    sendMessage(payload : JsonObject) : void {
        this.ws.send(JSON.stringify(payload));
    }

    async exec(command : string, args : any[] = []) : Promise<{}> {
        let msgId = uuid();
        return new Promise((resolve, reject) => {
            this.sendMessage({
                id: msgId,
                type: "command",
                command,
                args
            });

            let timeout = setTimeout(() => {
                if (this.requests[msgId]) {
                    this.requests[msgId].reject(new Error("EXEC_TIMEOUT"));
                }
            }, this.options.timeout || 5000);
            
            this.requests[msgId] = {
                reject,
                resolve,
                timeout
            };
        });
    }

    createCommand(command : string, handler : Function) : void {
        this.commands[command] = {
            handler
        };
    }

    removeCommand(command : string) : void {
        delete this.commands[command];
    }
}
