"use strict";

import { Component, IConfigReader, ILogger, Json, JsonObject, utils } from "merapi";
import { ISvxWebsocket, SvxSource, IRpc } from "./interface";

import * as WebSocket from "ws";
import Rpc from "./rpc";

const sleep = require("then-sleep");

import { IService, IServiceModule, Result, ServiceInfo } from "merapi-plugin-svx";

export default class SvxWebsocket extends Component implements ISvxWebsocket {
    private _connection : IRpc;

    private _moduleName : string = "websocket";
    private _priority : number = 3;
    private _version : number = 1;
    private _supportTransport : {[key: string] : boolean} = {
        api: true,
        command: true,
        topic: true,
        queue: true,
        discovery: true,
        log: true
    };

    constructor(
        private config : IConfigReader,
        private logger : ILogger,
        private service : IService
    ) {
        super();
    }

    async initialize() : Promise<void> {
        this.service.setDefaultTransport(this);

        await this.service.registerModule(this as any);

        return this.doConnect();
    }

    private async doConnect() : Promise<void> {
        const websocketUrl = this.config.default("websocket.url", "ws://localhost:8000");
        this._connection = new Rpc(new WebSocket(websocketUrl), this.config.default("websocket.options", {}));
    }

    async start() : Promise<void> {
        this._connection.createCommand("discovery.discover", async () => {
            this.broadcast(this.service.info());
        });

        this.handleBroadcast();

        setInterval(async () => {
            this.broadcast(this.service.info());
        }, parseInt(this.config.default("websocket.broadcastInterval", "5000")));
    }

    name() : string {
        return this._moduleName;
    }

    priority() : number {
        return this._priority;
    }

    info() : JsonObject {
        return {
            version: this._version,
            transport: this._supportTransport
        };
    }

    hasTransport(type : string) : boolean {
        if (Object.keys(this._supportTransport).indexOf(type) > -1)
            return this._supportTransport[type];
        return false;
    }

    getConnection() {
        return this._connection;
    }

    async publishTopic(service: string, topic: string, version : number, payload : Json, source : SvxSource) : Promise<void> {
        this._connection.exec("publishTopic", [ service, topic, version, payload, source ]);
    }

    subscribeTopic(service: string, topic: string, version: number, handler: (payload: Json, source: SvxSource) => Promise<void> | void) : void {
        let source = this.service.createSource() as SvxSource;

        this._connection.createCommand(`subscribeTopic.${service}.${topic}.${version}`, handler);
        this._connection.exec(`createTopic`, [ service, topic, version ]);
    }

    unsubscribeTopic(service: string, topic: string, version: number) : void {
        this._connection.exec(`unsubscribeTopic`, [service, topic, version]);
        this._connection.removeCommand(`subscribeTopic.${service}.${topic}.${version}`);
    }

    pushQueue(service: string, queue: string, version: number, payload: Json, source: SvxSource): void {
        this._connection.exec("pushQueue", [ service, queue, version, payload, source ]);
    }

    consumeQueue(service: string, queue: string, version: number, handler: (payload: Json, source: SvxSource) => Promise<void>): void {
        let source = this.service.createSource() as SvxSource;

        this._connection.createCommand(`consumeQueue.${service}.${queue}.${version}`, handler);
        this._connection.exec(`createQueue`, [service, queue, version]);
    }

    async callApi(service: string, method: string, version: number, args: Json[], source: SvxSource): Promise<any> {
        return await this._connection.exec("callApi", [ service,  method, version, args, source ]);
    }

    createApi(service: string, method: string, version: number, handler: (args: Json[], source: JsonObject) => Promise<Json>) : void {
        this._connection.createCommand(`handleApi.${service}.${method}.${version}`, handler);
        this._connection.exec("createApi", [ service, method, version ]);
    }

    async watchApiCall(service: string, method: string, version: number) : Promise<Result> {
        return;
    }

    async watchApiResult(service: string, method: string, version: number) : Promise<Result> {
        return;
    }

    async execCommand(service: string, command: string, args: Json[], source: JsonObject) : Promise<any> {
        return await this._connection.exec("execCommand", [ service,  command, args, source ]);
    }

    createCommand(service: string, command: string, handler: (args: Json[], source: JsonObject) => Promise<Json>) : void {
        this._connection.createCommand(`handleCommand.${service}.${command}`, handler);
        this._connection.exec("createCommand", [ service, command ]);
    }

    handleBroadcast() : void {
        this._connection.createCommand("discovery.broadcast", (info : ServiceInfo, source : SvxSource) => {
            this.service.handleBroadcast(info, source);
        });
    }

    async broadcast(info: any) : Promise<void> {
        this._connection.exec("discovery.broadcast", [ info, this.service.createSource() ]);
    }

    async discover(timeout: number = 500) : Promise<void> {
        this._connection.exec("discovery.discover", [{}]);
        return new Promise<void>((resolve, reject) => {
            setTimeout(resolve, timeout);
        });
    }

    subscribeLog(service: string, level: string, handler: (payload: Json, source: SvxSource) => Promise<void> | void) {
        let source = this.service.createSource() as SvxSource;

        this._connection.exec("subscribeLog", [ service, level ]);
        this._connection.createCommand(`handleLog.${service}.${level}`, handler);
    }

    log(service: string, component: string, method: string, level: string, message: Json): void {
        let source = this.service.createSource() as SvxSource;

        this._connection.exec("publishLog", [ service, component, method, level, message, source ]);
    }
}
